//Firmware rev1 for Alan Wilson's Nixie2 clock - Alan Wilson 2019

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <ArduinoOTA.h>
#include <WiFiUdp.h>

WiFiUDP Udp;
const unsigned int UdpRxPort = 4210;
const char* ssid = "SSID";
const char* password = "PASSWORD";
const char* Ehostname = "NixieESPBlue";

#define ledPin 2
#define srClockPin 14
#define srDataPin 13
#define srLatchPin 16

byte inBuffer[256]; //buffer for incoming UDP packet bytes

byte i=0;
byte nupdate=0;

unsigned long tcounter = 0;
unsigned long ndisplay = 0;

#include <Wire.h>
#include <simpleds3231.h>
DS3231 rtc;

#include <WiFiUdp.h>
#include "uxtime2.h"        //converts between timestamp and unix time
uxtime ux;

void setup() {
  pinMode(ledPin, OUTPUT);
  pinMode(srLatchPin,OUTPUT);
  pinMode(srDataPin,OUTPUT);
  pinMode(srClockPin,OUTPUT);
  Wire.begin(0,5); //start I2C bus
  Serial.begin(74880);
  Serial.println("Hello");
  
  digitalWrite(ledPin, LOW);

  WiFi.hostname(Ehostname);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  for(byte e=0; e<=9; e++){
    yield();
    nixie(e*111111L,1);
    delay(500);
  }
  nixie(0,0);
  if(WiFi.status() != WL_CONNECTED){
    int j=0;
    nixie(j,0);
    delay(1000);
    j++;
  }
  digitalWrite(ledPin, HIGH);
  ArduinoOTA.setPort(8266);
  ArduinoOTA.setHostname(Ehostname);
  ArduinoOTA.begin();
  Udp.begin(UdpRxPort);
  nixie(0,0);  
}

void loop() {
  ArduinoOTA.handle();
  rtc.getTime();
  if(millis()-tcounter>=10000){
    if(nupdate!=rtc.csecond()){
      nupdate=rtc.csecond();
      if(!(rtc.chour()>=0&&rtc.chour()<7)){
        nixie(rtc.csecond()+rtc.cminute()*100+rtc.chour()*10000,0);
      }else{
        nixie(111111L*(rtc.csecond()%10),(rtc.cminute()&1)*0xff);
      }
    }
  }
  int packetSize = Udp.parsePacket();
  if(packetSize){
    int len = Udp.read(inBuffer, sizeof(inBuffer));
    if(len > 0){
      switch(inBuffer[0]){
        case '-':
          ux.unix2datetime(ux.datetime2unix(rtc.cyear(),rtc.cmonth(),rtc.cday(),rtc.chour(),rtc.cminute(),rtc.csecond())-1);
          rtc.setTimeYear(ux.year()-2000);
          rtc.setTimeMonth(ux.month());
          rtc.setTimeDay(ux.day());
          rtc.setTimeHour(ux.hour());
          rtc.setTimeMinute(ux.minute());
          rtc.setTimeSecond(ux.second());
          rtc.validateClock();
          break;
        case '+':
          ux.unix2datetime(ux.datetime2unix(rtc.cyear(),rtc.cmonth(),rtc.cday(),rtc.chour(),rtc.cminute(),rtc.csecond())+1);
          rtc.setTimeYear(ux.year()-2000);
          rtc.setTimeMonth(ux.month());
          rtc.setTimeDay(ux.day());
          rtc.setTimeHour(ux.hour());
          rtc.setTimeMinute(ux.minute());
          rtc.setTimeSecond(ux.second());
          rtc.validateClock();
          break; 
        case 0x01: //set local time (yyyymmddhhmmss)
            rtc.setTimeYear(  inBuffer[1]);
            rtc.setTimeMonth( inBuffer[2]);
            rtc.setTimeDay(   inBuffer[3]);
            rtc.setTimeHour(  inBuffer[4]);
            rtc.setTimeMinute(inBuffer[5]);
            rtc.setTimeSecond(inBuffer[6]);
            rtc.validateClock();
            break;
        case 0x02: //display numbers
          ndisplay=(inBuffer[1])*100000
                  +(inBuffer[2])*10000
                  +(inBuffer[3])*1000
                  +(inBuffer[4])*100
                  +(inBuffer[5])*10
                  +(inBuffer[6]);
          nixie(ndisplay,inBuffer[7]);
          tcounter=millis();
          break;
        case 0x03: //display numbers (unsigned or signed long)
          ndisplay=(inBuffer[1]<<24)+(inBuffer[2]<<16)+(inBuffer[3]<<8)+(inBuffer[4]);
          nixie(ndisplay,inBuffer[5]);
          tcounter=millis();
          break;
      }
    }
  }
}

void nixie(long k, byte dp){ //displays data on tubes; k is the number, dp is a decimal point mask
  byte dig0=(k%10);
  byte dig1=(k%100)/10;
  byte dig2=(k%1000)/100;
  byte dig3=(k%10000)/1000;
  byte dig4=(k%100000)/10000;
  byte dig5=(k%1000000)/100000;
  byte dots=((dp&0b00100000)<<2)+((dp&0b00010000))+((dp&0b00001000)<<3)+((dp&0b00000100)<<1)+((dp&0b00000010)<<4)+((dp&0b00000001)<<2);
  shiftOut(srDataPin, srClockPin, MSBFIRST, dots);
  shiftOut(srDataPin, srClockPin, MSBFIRST, dig0*16+dig1);
  shiftOut(srDataPin, srClockPin, MSBFIRST, dig2*16+dig3);
  shiftOut(srDataPin, srClockPin, MSBFIRST, dig4*16+dig5);
  digitalWrite(srLatchPin,LOW);
  digitalWrite(srLatchPin,HIGH);
  yield();
}
